@@ -1,23 +1,14 @@
-"""Tests for pytest-cov.
 
-Known issues:
-
-- If py 2 then can have tx for any py 2, but problems if tx for py 3.
-
-- If py 3.0 then can have tx for py 3.0 / 3.1, but problems if tx for py 2.
-
-- If py 3.1 then can have tx for py 3.1, but problems if tx for py 2 or py 3.0.
-
-- For py 3.0 coverage seems to give incorrect results, it reports all
-  covered except the one line which it should have actually covered.
-  Issue reported upstream, also only problem with pass statement and
-  is fine with simple assignment statement.
-"""
-
-import py
 import os
 import sys
 
+import virtualenv
+
+import py
+import pytest
+import pytest_cov
+
+
 pytest_plugins = 'pytester', 'cov'
 
 SCRIPT = '''
@@ -28,21 +19,10 @@ def pytest_generate_tests(metafunc):
         metafunc.addcall()
 
 def test_foo():
-    version = sys.version_info[:2]
-    if version == (2, 4):
-        a = True
-    if version == (2, 5):
-        a = True
-    if version == (2, 6):
-        a = True
-    if version == (2, 7):
-        a = True
-    if version == (3, 0):
-        a = True
-    if version == (3, 1):
-        a = True
-    if version == (3, 2):
-        a = True
+    x = True
+    assert x
+    if sys.version_info[0] > 5:
+        assert False
 '''
 
 SCRIPT_CHILD = '''
@@ -65,14 +45,21 @@ def pytest_generate_tests(metafunc):
         metafunc.addcall(funcargs=dict(idx=i))
 
 def test_foo(idx):
-    out, err = subprocess.Popen([sys.executable, 'child_script.py', str(idx)], stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
+    out, err = subprocess.Popen(
+        [sys.executable, 'child_script.py', str(idx)],
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE).communicate()
+
+# there is a issue in coverage.py with multiline statements at
+# end of file: https://bitbucket.org/ned/coveragepy/issue/293
+pass
 '''
 
 SCRIPT_FUNCARG = '''
 import coverage
 
 def test_foo(cov):
-    assert isinstance(cov, coverage.control.coverage)
+    assert isinstance(cov, coverage.coverage)
 '''
 
 SCRIPT_FUNCARG_NOT_ACTIVE = '''
@@ -94,6 +81,17 @@ def test_run_target():
 '''
 
 
+SCRIPT_FAIL = '''
+def test_fail():
+    assert False
+
+'''
+
+SCRIPT_RESULT = '9 * 89%'
+CHILD_SCRIPT_RESULT = '6 * 100%'
+PARENT_SCRIPT_RESULT = '8 * 100%'
+
+
 def test_central(testdir):
     script = testdir.makepyfile(SCRIPT)
 
@@ -103,13 +101,26 @@ def test_central(testdir):
                                script)
 
     result.stdout.fnmatch_lines([
-            '*- coverage: platform *, python * -*',
-            'test_central * 20 * 70%*',
-            '*10 passed*'
-            ])
+        '*- coverage: platform *, python * -*',
+        'test_central * %s *' % SCRIPT_RESULT,
+        '*10 passed*'
+        ])
     assert result.ret == 0
 
 
+def test_no_cov_on_fail(testdir):
+    script = testdir.makepyfile(SCRIPT_FAIL)
+
+    result = testdir.runpytest('-v',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               '--no-cov-on-fail',
+                               script)
+
+    assert 'coverage: platform' not in result.stdout.str()
+    result.stdout.fnmatch_lines(['*1 failed*'])
+
+
 def test_dist_collocated(testdir):
     script = testdir.makepyfile(SCRIPT)
 
@@ -121,10 +132,10 @@ def test_dist_collocated(testdir):
                                script)
 
     result.stdout.fnmatch_lines([
-            '*- coverage: platform *, python * -*',
-            'test_dist_collocated * 20 * 70%*',
-            '*10 passed*'
-            ])
+        '*- coverage: platform *, python * -*',
+        'test_dist_collocated * %s *' % SCRIPT_RESULT,
+        '*10 passed*'
+        ])
     assert result.ret == 0
 
 
@@ -143,15 +154,16 @@ def test_dist_not_collocated(testdir):
                                script)
 
     result.stdout.fnmatch_lines([
-            '*- coverage: platform *, python * -*',
-            'test_dist_not_collocated * 20 * 70%*',
-            '*10 passed*'
-            ])
+        '*- coverage: platform *, python * -*',
+        'test_dist_not_collocated * %s *' % SCRIPT_RESULT,
+        '*10 passed*'
+        ])
     assert result.ret == 0
 
 
 def test_central_subprocess(testdir):
-    scripts = testdir.makepyfile(parent_script=SCRIPT_PARENT, child_script=SCRIPT_CHILD)
+    scripts = testdir.makepyfile(parent_script=SCRIPT_PARENT,
+                                 child_script=SCRIPT_CHILD)
     parent_script = scripts.dirpath().join('parent_script.py')
 
     result = testdir.runpytest('-v',
@@ -160,15 +172,16 @@ def test_central_subprocess(testdir):
                                parent_script)
 
     result.stdout.fnmatch_lines([
-            '*- coverage: platform *, python * -*',
-            'child_script * 6 * 100%*',
-            'parent_script * 7 * 100%*',
-            ])
+        '*- coverage: platform *, python * -*',
+        'child_script * %s *' % CHILD_SCRIPT_RESULT,
+        'parent_script * %s *' % PARENT_SCRIPT_RESULT,
+        ])
     assert result.ret == 0
 
 
 def test_dist_subprocess_collocated(testdir):
-    scripts = testdir.makepyfile(parent_script=SCRIPT_PARENT, child_script=SCRIPT_CHILD)
+    scripts = testdir.makepyfile(parent_script=SCRIPT_PARENT,
+                                 child_script=SCRIPT_CHILD)
     parent_script = scripts.dirpath().join('parent_script.py')
 
     result = testdir.runpytest('-v',
@@ -179,15 +192,16 @@ def test_dist_subprocess_collocated(testdir):
                                parent_script)
 
     result.stdout.fnmatch_lines([
-            '*- coverage: platform *, python * -*',
-            'child_script * 6 * 100%*',
-            'parent_script * 7 * 100%*',
-            ])
+        '*- coverage: platform *, python * -*',
+        'child_script * %s *' % CHILD_SCRIPT_RESULT,
+        'parent_script * %s *' % PARENT_SCRIPT_RESULT,
+        ])
     assert result.ret == 0
 
 
 def test_dist_subprocess_not_collocated(testdir, tmpdir):
-    scripts = testdir.makepyfile(parent_script=SCRIPT_PARENT, child_script=SCRIPT_CHILD)
+    scripts = testdir.makepyfile(parent_script=SCRIPT_PARENT,
+                                 child_script=SCRIPT_CHILD)
     parent_script = scripts.dirpath().join('parent_script.py')
     child_script = scripts.dirpath().join('child_script.py')
 
@@ -205,10 +219,10 @@ def test_dist_subprocess_not_collocated(testdir, tmpdir):
                                parent_script)
 
     result.stdout.fnmatch_lines([
-            '*- coverage: platform *, python * -*',
-            'child_script * 6 * 100%*',
-            'parent_script * 7 * 100%*',
-            ])
+        '*- coverage: platform *, python * -*',
+        'child_script * %s *' % CHILD_SCRIPT_RESULT,
+        'parent_script * %s *' % PARENT_SCRIPT_RESULT,
+        ])
     assert result.ret == 0
 
 
@@ -221,31 +235,21 @@ def test_empty_report(testdir):
                                script)
 
     result.stdout.fnmatch_lines([
-            '*- coverage: platform *, python * -*',
-            '*10 passed*'
-            ])
+        '*- coverage: platform *, python * -*',
+        '*10 passed*'
+        ])
     assert result.ret == 0
     matching_lines = [line for line in result.outlines if '%' in line]
     assert not matching_lines
 
 
 def test_dist_missing_data(testdir):
-    version = sys.version_info[:2]
-    if version == (2, 4):
-        exe = '/usr/local/py24/bin/python'
-    if version == (2, 5):
-        exe = '/usr/local/py25/bin/python'
-    if version == (2, 6):
-        exe = '/usr/local/py26/bin/python'
-    if version == (2, 7):
-        exe = '/usr/local/py27/bin/python'
-    if version == (3, 0):
-        exe = '/usr/local/py30/bin/python3.0'
-    if version == (3, 1):
-        exe = '/usr/local/py31/bin/python3.1'
-    if version == (3, 2):
-        exe = '/usr/local/py32/bin/python3.2'
-
+    venv_path = os.path.join(str(testdir.tmpdir), 'venv')
+    virtualenv.create_environment(venv_path)
+    if sys.platform == 'win32':
+        exe = os.path.join(venv_path, 'Scripts', 'python.exe')
+    else:
+        exe = os.path.join(venv_path, 'bin', 'python')
     script = testdir.makepyfile(SCRIPT)
 
     result = testdir.runpytest('-v',
@@ -256,8 +260,8 @@ def test_dist_missing_data(testdir):
                                script)
 
     result.stdout.fnmatch_lines([
-            '*- coverage: failed slaves -*'
-            ])
+        '*- coverage: failed slaves -*'
+        ])
     assert result.ret == 0
 
 
@@ -270,10 +274,10 @@ def test_funcarg(testdir):
                                script)
 
     result.stdout.fnmatch_lines([
-            '*- coverage: platform *, python * -*',
-            'test_funcarg * 3 * 100%*',
-            '*1 passed*'
-            ])
+        '*- coverage: platform *, python * -*',
+        'test_funcarg * 3 * 100%*',
+        '*1 passed*'
+        ])
     assert result.ret == 0
 
 
@@ -284,8 +288,8 @@ def test_funcarg_not_active(testdir):
                                script)
 
     result.stdout.fnmatch_lines([
-            '*1 passed*'
-            ])
+        '*1 passed*'
+        ])
     assert result.ret == 0
 
 
@@ -300,8 +304,160 @@ def test_multiprocessing_subprocess(testdir):
                                script)
 
     result.stdout.fnmatch_lines([
-            '*- coverage: platform *, python * -*',
-            'test_multiprocessing_subprocess * 8 * 100%*',
-            '*1 passed*'
-            ])
+        '*- coverage: platform *, python * -*',
+        'test_multiprocessing_subprocess * 8 * 100%*',
+        '*1 passed*'
+        ])
     assert result.ret == 0
+
+
+MODULE = '''
+def func():
+    return 1
+
+'''
+
+CONFTEST = '''
+
+import mod
+mod.func()
+
+'''
+
+BASIC_TEST = '''
+
+def test_basic():
+    x = True
+    assert x
+
+'''
+
+CONF_RESULT = 'mod * 2 * 100% *'
+
+
+def test_cover_conftest(testdir):
+    testdir.makepyfile(mod=MODULE)
+    testdir.makeconftest(CONFTEST)
+    script = testdir.makepyfile(BASIC_TEST)
+    result = testdir.runpytest('-v',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+    assert result.ret == 0
+    result.stdout.fnmatch_lines([CONF_RESULT])
+
+
+def test_cover_conftest_dist(testdir):
+    testdir.makepyfile(mod=MODULE)
+    testdir.makeconftest(CONFTEST)
+    script = testdir.makepyfile(BASIC_TEST)
+    result = testdir.runpytest('-v',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               '--dist=load',
+                               '--tx=2*popen',
+                               script)
+    assert result.ret == 0
+    result.stdout.fnmatch_lines([CONF_RESULT])
+
+
+COVERAGERC = '''
+[report]
+# Regexes for lines to exclude from consideration
+exclude_lines =
+    raise NotImplementedError
+
+'''
+
+EXCLUDED_TEST = '''
+
+def func():
+    raise NotImplementedError
+
+def test_basic():
+    x = True
+    assert x
+
+'''
+
+EXCLUDED_RESULT = '4 * 100% *'
+
+
+def test_coveragerc(testdir):
+    testdir.makefile('', coveragerc=COVERAGERC)
+    script = testdir.makepyfile(EXCLUDED_TEST)
+    result = testdir.runpytest('-v',
+                               '--cov-config=coveragerc',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+    assert result.ret == 0
+    result.stdout.fnmatch_lines(['test_coveragerc * %s' % EXCLUDED_RESULT])
+
+
+def test_coveragerc_dist(testdir):
+    testdir.makefile('', coveragerc=COVERAGERC)
+    script = testdir.makepyfile(EXCLUDED_TEST)
+    result = testdir.runpytest('-v',
+                               '--cov-config=coveragerc',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               '-n', '2',
+                               script)
+    assert result.ret == 0
+    result.stdout.fnmatch_lines(
+        ['test_coveragerc_dist * %s' % EXCLUDED_RESULT])
+
+
+CLEAR_ENVIRON_TEST = '''
+
+import os
+
+def test_basic():
+    os.environ.clear()
+
+'''
+
+
+def test_clear_environ(testdir):
+    script = testdir.makepyfile(CLEAR_ENVIRON_TEST)
+    result = testdir.runpytest('-v',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+    assert result.ret == 0
+
+
+SCRIPT_SIMPLE = '''
+
+def test_foo():
+    assert 1 == 1
+    x = True
+    assert x
+
+'''
+
+SCRIPT_SIMPLE_RESULT = '4 * 100%'
+
+
+@pytest.mark.skipif('sys.platform == "win32"')
+def test_dist_boxed(testdir):
+    script = testdir.makepyfile(SCRIPT_SIMPLE)
+
+    result = testdir.runpytest('-v',
+                               '--cov=%s' % script.dirpath(),
+                               '--boxed',
+                               script)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_dist_boxed * %s*' % SCRIPT_SIMPLE_RESULT,
+        '*1 passed*'
+        ])
+    assert result.ret == 0
+
+
+def test_not_started_plugin_does_not_fail(testdir):
+    plugin = pytest_cov.CovPlugin(None, None, start=False)
+    plugin.pytest_sessionfinish(None, None)
+    plugin.pytest_terminal_summary(None)
